function heater_mpcsim()
% Example control of a fired heater.

% Load the control package

if mpctools.isOctave()
    pkg('load', 'control');
end

% Initialize random seed

randn('state', 927);

% Define system sizes.

Nx   = 11;           % number of states
Nu   = 3;            % number of inputs
Ny   = 5;            % number of outputs
Nid  = 5;            % number of integrating disturbances
Nd   = 2;            % number of measured disturbances.
Nw   = Nx + Nid;     % number of augmented states
Nv   = Ny;           % number of output measurements
Nt   = 60;           % controller prediction horizon
Nmhe = 60;           % MHE Estimation horizon.
Ns   = 2*Ny;         % number of slack variables for soft output constraints

% Define sample time in min.

Delta = 1;

% Define plot windows

Npast = 120;   % Length of past to plot
Nfuture = 60;  % Lenght of future to plot

% Load matrices for linear heater model

load HeaterModelCont;

% Define parameters for heater

u0 = [100; 100; 95];
x0 = zeros(Nx,1);
y0 = [750; 200; 0; 882; 882];
d0 = [540;540];

% Define hard state bounds for range of model validity. The desired bounds
% xlb and xub are included elsewhere as soft constraints.

xmax = 1e6*ones(Nx,1);
xmin = -1e6*ones(Nx,1);

% Package up parameters.

pars = mpcsim.captureworkspace();
pars.ode = @(x, u, d) ode_pars(x, u, d, pars);

% Define initial conditions for x, u, y, and d.

pars.x_init = x0;
pars.u_init = u0;
pars.y_init = y0;
pars.d_init = d0;
pars.dmodel = {}; 

% Set MHE data for y and u.

pars.y    = repmat(pars.y_init, 1, Nmhe + 1);
pars.u    = repmat(pars.u_init, 1, Nmhe);
pars.dhat = zeros(Nid, 1);

% Save a history of xhat for use with the prior.

histstruct = struct('xhat', repmat(pars.x_init, 1, Npast), ...
                    'dhat', repmat(zeros(Nid, 1), 1, Npast));

% Create variables. First arguments are (short name, description, defaults)

MV1 = mpcsim.MVObject('FC1.SP', 'FC1.SP (BPH)', true(), ...
    'pltmin', 80.0, 'pltmax', 140.0, 'min', 85.0, 'max', 135.0, ...
    'ssrvalue', 0.001, 'svalue', 10.0, 'rvalue', 0.001, ...
    'roclim', 100.0, 'value', pars.u_init(1), 'dnoise', 0.01);

MV2 = mpcsim.MVObject('FC2.SP', 'FC2.SP (BPH)', true(), ...
    'pltmin', 80.0, 'pltmax', 140.0, 'min', 85.0, 'max', 135.0, ...
    'ssrvalue', 0.001, 'svalue', 10.0, 'rvalue', 0.001, ...
    'roclim', 100.0, 'value', pars.u_init(2), 'dnoise', 0.01);

MV3 = mpcsim.MVObject('FG.SP', 'FG.SP (MSCFH)', true(), ...
    'pltmin', 90.0, 'pltmax', 105.0, 'min', 92.0, 'max', 103.0, ...
    'ssrvalue', 0.001, 'svalue', 1000.0, 'rvalue', 0.01, ...
    'roclim', 100.0, 'value', pars.u_init(3), 'dnoise', 0.001);
    
DV1 = mpcsim.DVObject('TI1', 'TI1.PV (F)', true(), ...
    'pltmin', 530, 'pltmax', 560, 'value', pars.d_init(1), ...
    'dnoise', 0.01, 'noise', 0.01);

DV2 = mpcsim.DVObject('TI2', 'TI2.PV (F)', true(), ...
    'pltmin', 530, 'pltmax', 560, 'value', pars.d_init(2), ...
    'dnoise', 0.01, 'noise', 0.01);

XV1 = mpcsim.XVObject('S1', 'xv1', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(1), 'estvalue', pars.x_init(1), 'noise', 0.0);
  
XV2 = mpcsim.XVObject('S2', 'xv2', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(2), 'estvalue', pars.x_init(2), 'noise', 0.0);
  
XV3 = mpcsim.XVObject('S3', 'xv3', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(3), 'estvalue', pars.x_init(3), 'noise', 0.0);
  
XV4 = mpcsim.XVObject('S4', 'xv4', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(4), 'estvalue', pars.x_init(4), 'noise', 0.0);
  
XV5 = mpcsim.XVObject('S5', 'xv5', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(5), 'estvalue', pars.x_init(5), 'noise', 0.0);
  
XV6 = mpcsim.XVObject('S6', 'xv6', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(6), 'estvalue', pars.x_init(6), 'noise', 0.0);
  
XV7 = mpcsim.XVObject('S7', 'xv7', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(7), 'estvalue', pars.x_init(7), 'noise', 0.0);
  
XV8 = mpcsim.XVObject('S8', 'xv8', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(8), 'estvalue', pars.x_init(8), 'noise', 0.0);
  
XV9 = mpcsim.XVObject('S9', 'xv9', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(9), 'estvalue', pars.x_init(9), 'noise', 0.0);
  
XV10 = mpcsim.XVObject('S10', 'xv10', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(10), 'estvalue', pars.x_init(10), 'noise', 0.0);
  
XV11 = mpcsim.XVObject('S11', 'xv11', true(), ...
    'pltmin', -200, 'pltmax', 200, 'dnoise',  0.01,  ...
    'value', pars.x_init(11), 'estvalue', pars.x_init(11), 'noise', 0.0);
  
cvslack = {'lbslack', 1000, '.min', 0, '.name', 'LB Viol. Penalty', ...
           'ubslack', 1000, '.min', 0, '.name', 'UB Viol. Penalty'};

cvnosp = {'setpoint', NaN(), '.visible', false(), 'ssqvalue', NaN()};

CV1 = mpcsim.CVObject('TO', 'TO.PV (F)', true(), ...
    'pltmin', 745, 'pltmax', 765, 'min', 740, 'max', 760, ...
    'qvalue', 100.0, 'mnoise', 0.01, 'dnoise', 0.01, 'value',pars.y_init(1), ...
    'estvalue', pars.y_init(1), 'setpoint', pars.y_init(1), ...
    'ssqvalue', 100.0, cvslack{:}, 'noise', 0.01);

CV2 = mpcsim.CVObject('FO', 'FO.PV (BPH)', true(), ...
    'pltmin', 190.0, 'pltmax', 270.0, 'min', 190.0, 'max', 260.0, ...
    'qvalue', 1.0, 'mnoise', 0.01, 'dnoise', 0.01, 'value', pars.y_init(2), ...
    'estvalue', pars.y_init(2),  'setpoint', pars.y_init(2), ...
     'ssqvalue', 1.0, cvslack{:}, 'noise', 0.01);

CV3 = mpcsim.CVObject('DT', 'DT.PV (F)', true(), ...
    'pltmin', -10.0, 'pltmax', 10.0, 'min', -5.0, 'max', 5.0, ...
    'qvalue', 0.01, 'mnoise', 0.01, 'dnoise', 0.01, 'value', pars.y_init(3), ...
    'estvalue', pars.y_init(3), 'setpoint', pars.y_init(3), ...
    'ssqvalue', 0.01, cvslack{:}, 'noise', 0.01);

CV4 = mpcsim.CVObject('TS1', 'TS1.PV (F)', true(), ...
    'pltmin', 870.0, 'pltmax', 930.0, 'min', 870.0, 'max', 918.0, ...
    'qvalue', 0.001, 'mnoise', 0.01, 'dnoise', 0.01, 'value', pars.y_init(4), ...
    'estvalue', pars.y_init(4), 'setpoint', pars.y_init(4), ...
    'ssqvalue', 0.001, cvslack{:}, 'noise', 0.01);
    
CV5 = mpcsim.CVObject('TS2', 'TS2.PV (F)', true(), ...
    'pltmin', 870.0, 'pltmax', 930.0, 'min', 870.0, 'max', 918.0, ...
    'qvalue', 0.001, 'mnoise', 0.01, 'dnoise', 0.01, 'value', pars.y_init(5), ...
    'estvalue', pars.y_init(5), 'setpoint', pars.y_init(5), ...
    'ssqvalue', 0.001, cvslack{:}, 'noise', 0.01);
    
options = mpcsim.Option('Options', ...
    'noisefactor', 0, '.name', 'Noise Factor', ...
    'priorweight', 0, '.name', 'MHE Prior Weight (Pe)', '.min', 0, ...
    'dmodnum', 2, '.name', 'Disturbance Model Number', '.type', 'int', '.min', 1, '.max', 3);

%    'linmpc', true(), '.name', 'Linear MPC', '.type', 'bool', ...
%    'statefeedback', false(), '.name', 'State Feedback', '.type', 'bool', ...

% Don't want XV plots for this example   
    
% XVplots = false();
    
% Create gui.

layout = {'MV1', 'CV1'; 'MV2', 'CV2'; 'MV3', 'CV3'; 'DV1', 'CV4'; 'DV2', 'CV5'};
%layout = {'MV1', 'CV1'; 'MV2', 'CV2'; 'MV3', 'CV3'; 'DV1', 'CV4'; 'DV2', 'CV5'; ...
%          'XV1', 'XV2'; 'XV3', 'XV4'; 'XV5', 'XV6'; 'XV7', 'XV8'; 'XV9', 'XV10'; ...
%          'XV11', ''};
%layout = {'MV1', 'CV1', 'XV1', 'XV2'; ...
%          'MV2', 'CV2', 'XV3', 'XV4'; ...
%          'MV3', 'CV3', 'XV5', 'XV6'; ...
%          'DV1', 'CV4', 'XV7', 'XV8'; ...
%          'DV2', 'CV5', 'XV9', 'XV10'; ...
%          ' ',   ' ',    ' ',  'XV11'};
simcon = mpcsim.SimCon(@simstep, {MV1, MV2, MV3}, {CV1, CV2, CV3, CV4, CV5}, options, ...
                       'xvs', {XV1, XV2, XV3, XV4, XV5, XV6, XV7, XV8, XV9, XV10, XV11}, ...
                       'dvs', {DV1, DV2}, 'data', pars, ...
                       'Npast', Npast, 'Nfuture', Nfuture, 'layout', layout, ...
                       'history', histstruct);
%                       'history', histstruct, 'XVplots', XVplots);

simcon.step(); % Step once to initialize everything.
simcon.step();
simcon.step();
simcon.mainloop();

end%function

% Define ODE function for the heater

function dxdt = ode_pars(x, u, d, pars)

    % ODE model for heater

    uf = [u;d];
    uf0 = [pars.u0;pars.d0];
    dxdt = pars.A*(x - pars.x0) + pars.B*(uf - uf0);
    
end%function

% Define simulation function

function xsim = rk4sim(ode, Delta, x, u)

    % Simulates an ODE with disturbance model using RK4.

    Nsim = size(u, 2);
    xsim = NaN(length(x), Nsim + 1);
    xsim(:,1) = x;
    for i = 1:Nsim
        f = @(x) ode(x, u(:,i));
        xsim(:,i + 1) = mpctools.rk4_(Delta, f, xsim(:,i));
    end

end%function

% Define steady-state cost function

function cost = targetcost(y, ysp, Q, u, usp, R, s, p)

    % Steady-state target cost. s is a vector of state constraint slacks.

    dy = y - ysp;
    du = u - usp;
    cost = dy'*Q*dy + du'*R*du + p'*s;

end%function

% Define dynamic stage cost function

function cost = stagecost(x, xsp, Q, u, usp, R, Du, S, s, p)

    % Stage cost. S is rate-of-change penalty for u. s is vector of state
    % constraint slacks. p is penalty vector for output constraints.

    cost = targetcost(x, xsp, Q, u, usp, R, s, p) + Du'*S*Du;

end%function

% Define soft output constraint slack function

function con = slackoutputs(y, s, ylb, yub)

    % Slacked output constraints.

    Ny = length(y);
    slb = s(1:Ny);
    sub = s((Ny + 1):end);
    con = [
        y - yub - sub;
        ylb - y - slb;
    ];

end%function

% Define simulation functions

function simstep(simcon, iteration, stale)

    % Runs one step of the heater example

    fprintf('Iteration %4d -----------------------------------\n', iteration);
    mpc = import_mpctools();
    
    % Unpack stuff from simulation container.

    parnames = {'xscale', 'yscale', 'uscale', 'dscale', 'B', 'C', 'Nx', 'Ny', ...
                'Nu', 'Nd', 'Nid', 'Ns', 'Nw', 'Nv', 'Nt', 'Nmhe', 'ode', ...
                'xmin', 'xmax', 'Delta', 'y', 'u', 'x_init', 'y_init', ...
                'u_init', 'sim', 'controller', 'mhe', 'sstarg', 'linmodel', ...
                'd_init', 'dhat'};
    [xscale, yscale, uscale, dscale, B, C, Nx, Ny, Nu, Nd, Nid, Ns, Nw, Nv, ...
        Nt, Nmhe, ode, xmin, xmax, Delta, y, u, x_init, y_init, u_init, ...
        sim, controller, mhe, sstarg, linmodel, d_init, dhat] ...
        = mpctools.structdeal({simcon.data, []}, parnames{:});

    % Get option flag values

%    nonlinmpc     = ~simcon.Options.get('linmpc');
%    statefeedback = simcon.Options.get('statefeedback');
    dmodnum       = simcon.Options.get('dmodnum');
    
    % Initialize data in .mat file
    
    if (iteration == 0)
        udata = [];
        ydata = [];
        yspdata = [];
        uubdata = [];
        ulbdata = [];
        yubdata = [];
        ylbdata = [];
        save('heaterData','udata','ydata','yspdata','uubdata','ulbdata','yubdata','ylbdata');
    end
    
    % Construct new disturbance model is necessary

    if (iteration == 0)
        olddmodnum = 0;
    else
        olddmodnum = simcon.data.dmodnum;
    end  
    
    if ~isequal(dmodnum, olddmodnum)

        switch dmodnum

        case 1 % Output disturbances on TO.PV, FO.PV, DT.PV (Not enough integrating disturbances)

            Nid = 3;
            Bd = zeros(Nx, Nid);
            Cd = [1 0 0; 0 1 0; 0 0 1;0 0 0;0 0 0];
            dmodel = {'y1','y2','y3'};
            dhat = zeros(Nid,1);
            stale = true();
            
        case 2 % Output disturbances on TO.PV, FO.PV, DT.PV, TS1.PV, TS2.PV (Offset free)
    
            Nid = 5;
            Bd = zeros(Nx, Nid);
            Cd = [1 0 0 0 0; 0 1 0 0 0; 0 0 1 0 0; 0 0 0 1 0; 0 0 0 0 1];
            dmodel = {'y1','y2','y3','y4','y5'};
            dhat = zeros(Nid,1);
            stale = true();
    
    
        case 3 % Input disturbances on FC1.SP, FC2.SP, FG.SP, output disturbances on TO.PV and FO.PV (Offset free)
    
            Nid = 5;
            Bd = zeros(Nx, Nid);
            Bd(:,1) = B(:,1);
            Bd(:,2) = B(:,2);
            Bd(:,3) = B(:,3);
            Cd = zeros(Ny, Nid);
            Cd(1,:) = [0 0 0 1 0];
            Cd(2,:) = [0 0 0 0 1];
            dmodel = {'u1','u2','u3','y1','y2'};
            dhat = zeros(Nid,1);
            stale = true();
    
        end

        simcon.setdisturbancemodel(dmodel);
        fprintf('  Choosing disturbance model [%s]\n', ...
                mpctools.row2str(dmodel)');
        simcon.data.dmodel = dmodel;
        simcon.data.dmodnum = dmodnum;
        simcon.data.Bd = Bd;
        simcon.data.Cd = Cd;
        simcon.data.Nid = Nid;
        simcon.data.Nw = Nx + Nid;
        Bd
        Cd
        
    end
    
    % Grab bounds and penalty coefficients

    ulb = simcon.getvec('MVs', 'min');
    uub = simcon.getvec('MVs', 'max');
    ylb = simcon.getvec('CVs', 'min');
    yub = simcon.getvec('CVs', 'max');
    dulim = simcon.getvec('MVs', 'roclim');
    
    xsys = simcon.getvec('XVs', 'value');
    usys = simcon.getvec('MVs', 'value');
    mpcpar = struct();
    Qy = diag(simcon.getvec('CVs', 'qvalue'));
    mpcpar.Q = C'*Qy*C;
    mpcpar.P = mpcpar.Q;
    mpcpar.R = diag(simcon.getvec('MVs', 'rvalue'));
    mpcpar.S = diag(simcon.getvec('MVs', 'svalue'));
    mpcpar.p = [ ...
        simcon.getvec('CVs', 'lbslack');
        simcon.getvec('CVs', 'ubslack');
    ];
    mpcpar.xsp = zeros(Nx, 1); % Gets set later.
    mpcpar.usp = zeros(Nu, 1); % Gets set later.
    mpcpar.uprev = usys;
    mpcpar.ylb = ylb;
    mpcpar.yub = yub;
    
    mpcpar.C = C;
    mpcpar.Bd = simcon.data.Bd;
    mpcpar.Cd = simcon.data.Cd;
    mpcpar.dhat = zeros(Nid, 1);
    
    sstargpar = rmfield(mpcpar, {'P', 'S', 'uprev'});
    sstargpar.Q = Qy;
    sstargpar.R = diag(simcon.getvec('MVs', 'ssrvalue'));
%    sstargpar.ysp = zeros(Ny, 1); % Gets set later.
    sstargpar.ysp = simcon.getvec('CVs','setpoint');
    
    mhepar = struct();
    mhepar.Qinv = eye(Nx+Nid);
    mhepar.Rinv = diag(1./(simcon.getvec('CVs', 'mnoise').^2));
    mhepar.Pinv = mhepar.Qinv; % Will be set later.
    mhepar.xbar = [simcon.history.xhat.get(-Nmhe);
                   simcon.history.dhat.get(-Nmhe)];
    lxbar = length(mhepar.xbar);
    Nw = simcon.data.Nw;
    if (lxbar > Nw)
        mhepar.xbar = mhepar.xbar((1:Nw),:);
    end
    if (lxbar < Nw)
        ldiff = Nw - lxbar;
        xfill = zeros(ldiff,1);
        mhepar.xbar((lxbar+1):Nw,:) = xfill;
    end   
    mhepar.C = C;
    mhepar.Bd = simcon.data.Bd;
    mhepar.Cd = simcon.data.Cd;
        
    % Get disturbance model
        
    Bd = simcon.data.Bd;
    Cd = simcon.data.Cd;
    Nid = simcon.data.Nid;
    Nw = simcon.data.Nw;

    % Rebuild items if necessary

    if stale
        fprintf('  Building controllers ');
        
        
        % Make a one-step simulator and a multi-step simulator

        simargs = {};
        sim = mpc.getCasadiIntegrator(ode, Delta, [Nx, Nu, Nd], ...
                                      {'x', 'u', 'd'}, 'funcname', 'sim');
        simcon.data.sim = sim;
        fprintf('.');
        
        % Get functions for the controller

        ftarg = mpc.getCasadiFunc(@(x, u, Bd, dhat) ode(x, u, d_init) + Bd*dhat, ...
                                  {Nx, Nu, [Nx, Nid], Nid}, ...
                                  {'x', 'u', 'Bd', 'dhat'}, ...
                                 'funcname', 'f');
        fmpc = mpc.getCasadiFunc(@(x, u, Bd, dhat) ode(x, u, d_init) + Bd*dhat, ...
                                 {Nx, Nu, [Nx, Nid], Nid}, ...
                                 {'x', 'u', 'Bd', 'dhat'}, ...
                                 'rk4', true(), 'Delta', Delta, ...
                                 'funcname', 'f');
        fmhe = mpc.getCasadiFunc(@(x, u, Bd) [ode(x(1:Nx), u, d_init) ...
                                              + Bd*x((Nx + 1):end);
                                              zeros(Nid, 1)], ...
                                 {Nx + Nid, Nu, [Nx, Nid]}, ...
                                 {'x', 'u', 'Bd'}, ...
                                 'rk4', true(), 'Delta', Delta, ...
                                 'funcname', 'faug');
        
        htarg = mpc.getCasadiFunc(@(C, x, Cd, dhat) C*x + Cd*dhat, ...
                                  {[Ny, Nx], Nx, [Ny, Nid], Nid}, ...
                                  {'C', 'x', 'Cd', 'dhat'}, ...
                                  'funcname', 'h');
        hmpc = htarg;
        hmhe = mpc.getCasadiFunc(@(C, x, Cd) C*x(1:Nx) + Cd*(x((Nx + 1):end)), ...
                                 {[Ny, Nx], Nx + Nid, [Ny, Nid]}, ...
                                 {'C', 'x', 'Cd'}, 'funcname', 'haug');
        
        e = mpc.getCasadiFunc(@slackoutputs, [Ny, Ns, Ny, Ny], ...
                              {'y', 's', 'ylb', 'yub'}, 'funcname', 'e');
        
        ltarg = mpc.getCasadiFunc(@targetcost, ...
            {Ny, Ny,   [Ny,Ny], Nu, Nu, [Nu,Nu],Ns, Ns}, ...
            {'y','ysp','Q',    'u','usp','R',  's','p'}, ...
            'funcname', 'ltarg');
        lmpc = mpc.getCasadiFunc(@stagecost, ...
            {Nx, Nx,   [Nx,Nx], Nu, Nu, [Nu,Nu], Nu,[Nu,Nu], Ns, Ns}, ...
            {'x','xsp','Q',    'u','usp','R',   'Du','S',    's','p'}, ...
            'funcname', 'l');
        lmhe = mpc.getCasadiFunc(@(w, Qinv, v, Rinv) w'*Qinv*w + v'*Rinv*v, ...
                                 {Nw, [Nw, Nw], Nv, [Nv, Nv]}, ...
                                 {'w', 'Qinv', 'v', 'Rinv'}, 'funcname', 'lmhe');
        
        Vf = mpc.getCasadiFunc(@(x, xsp, P) (x - xsp)'*P*(x - xsp), ...
                               {Nx, Nx, [Nx, Nx]}, {'x','xsp','P'}, ...
                               'funcname', 'Vf');
        
        lx = mpc.getCasadiFunc(@(x, xbar, Pinv) (x - xbar)'*Pinv*(x - xbar), ...
                               {Nx + Nid, Nx + Nid, (Nx + Nid)*[1, 1]}, ...
                               {'x', 'xbar', 'Pinv'}, 'funcname', 'lx');
        fprintf('.');
        
        % Assemble controller

        N = struct('x', Nx, 'u', Nu, 'y', Ny, 's', Ns, 't', Nt);
        lb = struct('u', ulb, 'x', xmin);
        ub = struct('u', uub, 'x', xmax);
        guess = struct('u', repmat(mpcpar.usp, 1, Nt));
        controller = mpc.nmpc('f', fmpc, 'e', e, 'h', hmpc, 'l', lmpc, ...
                              'Vf', Vf, 'N', N, ...
                              'lb', lb, 'ub', ub, 'guess', guess, ...
                              'par', mpcpar, 'verbosity', 0);
        simcon.data.controller = controller;
        fprintf('.');
        
        % Assemble steady-state target finder

        N = struct('x', Nx, 'u', Nu, 'y', Ny, 's', Ns);
        lb = struct('u', ulb, 'x', xmin);
        ub = struct('u', uub, 'x', xmax);
        guess = struct('u', mpcpar.usp);
        sstarg = mpc.sstarg('f', ftarg, 'e', e, 'h', htarg, 'l', ltarg, ...
                            'N', N, 'lb', lb, 'ub', ub, 'guess', guess, ...
                            'par', sstargpar, 'discretef', false(), ...
                            'verbosity', 0);
        simcon.data.sstarg = sstarg;
        fprintf('.');
        
        % Assemble moving horizon estimator - disturbance model is output bias

        N = struct('x', Nx + Nid, 'y', Ny, 'u', Nu, 't', Nmhe);
        lb = struct('x', [xmin; -inf(Nid, 1)]);
        ub = struct('x', [xmax; inf(Nid, 1)]);
        mhe = mpc.nmhe('f', fmhe, 'h', hmhe, 'l', lmhe, 'lx', lx, ...
                       'y', y, 'u', u, 'N', N, 'lb', lb, 'ub', ub, ...
                       'par', mhepar, 'verbosity', 0, 'wadditive', true());
        simcon.data.mhe = mhe;
        fprintf('.');
        
        % Get linear model

        linmodel = struct('xlin', x_init, 'ulin', u_init, 'ylin', y_init);
        [linmodel.A, linmodel.B, linmodel.Bd] = mpc.getLinearizedModel(sim, ...
            {simcon.data.x_init, simcon.data.u_init, simcon.data.d_init}, ...
            {'A', 'B', 'Bd'}, 'deal', true());
        linmodel.C = C;
        simcon.data.linmodel = linmodel;    
       
        % All done
        
        fprintf('done\n');
        
    end
        
    % Advance actual system with noise

    noise = simcon.Options.get('noisefactor');
    usys = usys + noise*randn(Nu, 1).*simcon.getvec('u', 'noise');
    dsys = simcon.getvec('DVs', 'value') ...
           + noise*randn(Nd, 1).*simcon.getvec('d', 'noise');
    if iteration > 0
        xsys = full(sim(xsys, usys, dsys));
        xsys = xsys + noise*randn(Nx, 1).*simcon.getvec('x', 'noise');
    end
    ysys = C*(xsys - x_init) + y_init;
    ysys = ysys + noise*randn(Ny, 1).*simcon.getvec('y', 'noise');
    simcon.data.y = [simcon.data.y(:,2:end), ysys];
    simcon.data.u = [simcon.data.u(:,2:end), usys];
%    usys
%    xsys
%    ysys
    
    % Construct augmented linear system and compute Kalman Filter gain

    Aaug = [linmodel.A, mhepar.Bd; zeros(Nid, Nx), eye(Nid)];
    Gaug = eye(Nx + Nid);
    Baug = [linmodel.B; zeros(Nid, Nu)];
    Caug = [linmodel.C, mhepar.Cd];
    Qaug = diag(1./diag(mhepar.Qinv));
    Raug = diag(1./diag(mhepar.Rinv));
    % Raug(isinf(Raug)) = 1e12; % dlqe doesn't like infinite values here.
    [Laug, Paug] = dlqe(Aaug, Gaug, Caug, Qaug, Raug);

    % Test disturbance model for detectability
        
    Naug = size(Aaug,1);
    detec = rank([eye(Naug) - Aaug; Caug]);
    if detec < Nx + Nid
        fprintf(' * Augmented system is not detectable!\n')
    end
    
    % Test for not enough integrating disturbances
    
    if Nid < Ny
        fprintf(' * Not enough integrating disturbances!\n')
    end
    
%    if 
%
%        % Just take actual state.
%
%        xhat = xsys;
%        dhat = zeros(Nid, 1);
%        xmhe = NaN(Nx, Nmhe + 1);
%        dmhe = NaN(Nid, Nmhe + 1);

%    elseif nonlinmpc

        % Run MHE

        mhepar.Pinv = simcon.Options.get('priorweight')*mpctools.spdinv(Paug);
        mhepar.y = simcon.data.y;
        mhepar.u = simcon.data.u;
        mhe.par = mhepar;
        mhe.lb.x = repmat([xmin; -inf(Nid, 1)], 1, Nmhe + 1);
        mhe.ub.x = repmat([xmax; inf(Nid, 1)], 1, Nmhe + 1);
        if (stale || simcon.isfirsttimemhe)
            mhe.guess.x = repmat([xsys; zeros(Nid, 1)], 1, Nmhe + 1);
        end
        mhe.solve();
        fprintf('  Estimator: %s (Obj: %g)\n', mhe.status, mhe.obj);
        mhe.saveguess();
        
        xhat = mhe.var.x(1:Nx,end);
        dhat = mhe.var.x((Nx + 1):end,end);
        xmhe = mhe.var.x(1:Nx,:);
        dmhe = mhe.var.x((Nx + 1):end,:);

        if simcon.isfirsttimemhe
          simcon.isfirsttimemhe = false();
        end
        
%    else
%
%        if ~simcon.isfirsttimemhe
%          simcon.isfirsttimemhe = true();
%        end
%    
%        % Use Kalman filter.
%
%        xhat = [simcon.getvec('x', 'estvalue'); dhat];
%        xlin = [linmodel.xlin; zeros(Nid, 1)];
%        ulin = linmodel.ulin;
%        ylin = linmodel.ylin;
%        xhatm = Aaug*(xhat - xlin) + Baug*(usys - ulin) + xlin;
%        yhatm = Caug*(xhatm - xlin) + ylin;
%        xhat = xhatm + Laug*(ysys - yhatm);
%        
%        dhat = xhat((Nx + 1):end);
%        xhat = xhat(1:Nx);
%        xmhe = NaN(Nx, Nmhe + 1);
%        dmhe = NaN(Nid, Nmhe + 1);
%
%    end

    simcon.history.xhat.pushpop(xhat);
    dhata = dhat;
    if (Nid == 2)
        dhata = [dhat; 0.0];
    end
    simcon.history.dhat.pushpop(dhata); 
    simcon.data.dhat = dhat;
    yhat = C*xhat + simcon.data.Cd*dhat;
    
    % Update plots
    
    simcon.setvec('x', 'value', xsys);
    simcon.setvec('y', 'value', ysys);
    simcon.setvec('x', 'estvalue', xhat);
    simcon.setvec('y', 'estvalue', yhat);
    if isequal(simcon.data.dmodel{2}, 'd1')
        simcon.setvec('d', 'estvalue', dhat(2));
    end
    
    % Run controller and simulate predictions
    
    multisim = @(x, u) rk4sim(@(x, u) ode(x, u, d_init) + simcon.data.Bd*dhat, ...
                                          Delta, x, u);
    if simcon.isclosedloop
        
%        fprintf('  isfirstimeclosedloop = %4d \n', simcon.isfirsttimeclosedloop);
%        fprintf('  stale                = %4d \n', stale);

        % Steady-state target finder
        
        if (stale || simcon.isfirsttimeclosedloop)
            sstarg.guess.x = xsys;
            sstarg.guess.u = usys;
        end
%        csp = simcon.CVs{1}.get('setpoint');
%        hsp = simcon.CVs{3}.get('setpoint');
        
%        if nonlinmpc
            % Use nonlinear target finder.
            sstarg.par = sstargpar;
%            sstarg.par.ylb(1) = csp;
%            sstarg.par.yub(1) = csp;
%            sstarg.par.ysp(1) = csp;
%            sstarg.par.ylb(3) = hsp;
%            sstarg.par.yub(3) = hsp;
%            sstarg.par.ysp(3) = hsp;
            sstarg.par.dhat = dhat;

            sstarg.lb.u = ulb;
            sstarg.ub.u = uub;
            sstarg.solve();
            fprintf('  Target Finder: %s (Obj: %g)\n', sstarg.status, sstarg.obj);
            sstarg.saveguess();
            
            xtarg = sstarg.var.x;
            utarg = sstarg.var.u;
%        else
%            % Use linear target finder.
%            H = [1, 0, 0; 0, 0, 1];
%            Ginv = [eye(Nx) - linmodel.A, -linmodel.B; ...
%                    H*linmodel.C, zeros(size(H,1), Nu)]\eye(Nx + Nu);
%            ysp = [csp; 0; hsp] - linmodel.ylin;
%            targ = Ginv*[mhepar.Bd*dhat; H*(ysp - mhepar.Cd*dhat)];
%            xtarg = targ(1:Nx) + linmodel.xlin;
%            utarg = targ((Nx + 1):end) + linmodel.ulin;
%        end
        
        % Controller
        
        if (stale || simcon.isfirsttimeclosedloop)
            controller.guess.x = repmat(xsys, 1, Nt + 1);
        end
        controller.par = mpcpar;
        controller.par.xsp = xtarg;
        controller.par.usp = utarg;
        controller.par.dhat = dhat;
        controller.lb.u = repmat(ulb, 1, Nt);
        controller.ub.u = repmat(uub, 1, Nt);
        controller.lb.Du = repmat(-dulim, 1, Nt);
        controller.ub.Du = repmat(dulim, 1, Nt);
%        if nonlinmpc
            controller.fixvar('x', 1, xhat);
            controller.solve();
            fprintf('  Controller: %s (Obj: %g)\n', controller.status, ...
                    controller.obj);
            controller.saveguess();
            usys = controller.var.u(:,1);
            usim = controller.var.u;
            xsim = controller.var.x;
%        else
%            Aaug = blkdiag(linmodel.A, zeros(Nu, Nu));
%            Baug = [linmodel.B; eye(Nu)];
%            Qaug = blkdiag(mpcpar.Q, mpcpar.S);
%            Raug = mpcpar.R + mpcpar.S;
%            Maug = [zeros(Nx, Nu); -mpcpar.S];
%            Kaug = -dlqr(Aaug, Baug, Qaug, Raug, Maug);
%            Kx = Kaug(:,1:Nx);
%            Ku = Kaug(:,(Nx + 1):end);
%            usim = NaN(Nu, Nt);
%            xsim = NaN(Nx, Nt + 1);
%            xsim(:,1) = xhat;
%            uprev = mpcpar.uprev;
%            for i = 1:Nt
%                usim(:,i) = Kx*(xsim(:,i) - xtarg) + Ku*(uprev - utarg) + utarg;
%                usim(:,i) = min(max(usim(:,i), ulb), uub);
%                uprev = usim(:,i);
%                xsim(:,i + 1) = linmodel.A*(xsim(:,i) - xtarg)...
%                                + linmodel.B*(usim(:,i) - utarg) ...
%                                + xtarg;
%            end
%            usys = usim(:,1);  
%        end
        
        % Reset flag for first time closed loop
        
        if simcon.isfirsttimeclosedloop
            simcon.isfirsttimeclosedloop = false();
        end
    else
        
        % Set flag for first time closed loop
        
        if ~simcon.isfirsttimeclosedloop
            simcon.isfirsttimeclosedloop = true();
        end

        % Make forecast assuming constant input.
        
        usim = repmat(usys, 1, Nt);
        xsim = multisim(xhat, usim);
        
        xtarg = NaN(Nx, 1);
        utarg = NaN(Nu, 1);
    end
    
    % Update plots
    
    simcon.setvec('u', 'value', usys);
    ysim = bsxfun(@plus, C*xsim, simcon.data.Cd*dhat);
    
    % Save data to file heaterData.mat
    
    load('heaterData','udata','ydata','yspdata','uubdata','ulbdata','yubdata','ylbdata');
    udata   = [udata usys];
    ydata   = [ydata ysys];
    yspdata = [yspdata sstargpar.ysp];
    uubdata = [uubdata uub];
    ulbdata = [ulbdata ulb];
    yubdata = [yubdata yub];
    ylbdata = [ylbdata ylb];
    save('heaterData','udata','ydata','yspdata','uubdata','ulbdata','yubdata','ylbdata');
  
    % Simulate constant-u system
       
    uconst = repmat(usys, 1, Nt);
    xconst = multisim(xhat, uconst);
    yconst = bsxfun(@plus, C*xconst, simcon.data.Cd*dhat);
    
    % Update forecasts
       
    simcon.setvec('x', 'setpoint', xtarg);
    simcon.setvec('u', 'setpoint', utarg);
    simcon.setpred('x', xsim);
    simcon.setpred('y', ysim);
    simcon.setpred('u', usim);
    simcon.setpred('yconstu', yconst);
    simcon.setpred('xmhe', xmhe, 'past');
    if isequal(simcon.data.dmodel{2}, 'd1')
        simcon.setpred('dmhe', dmhe(2,:), 'past');
    end
    
    % Done
    fprintf('--------------------------------------------------\n');
    
end%function

